@@ -16,7 +16,7 @@ operations related to Catalog(Collections and Contents).
 
 from idds.common import exceptions
 from idds.common.constants import (CollectionType, CollectionStatus, CollectionLocking,
-                                   CollectionRelationType, ContentStatus)
+                                   CollectionRelationType, ContentStatus, ContentRelationType)
 from idds.orm.base.session import read_session, transactional_session
 from idds.orm import (transforms as orm_transforms,
                       collections as orm_collections,
@@ -314,7 +314,17 @@ def get_contents(coll_scope=None, coll_name=None, request_id=None, workload_id=N
 
     coll_ids = [coll['coll_id'] for coll in collections]
     if coll_ids:
-        rets = orm_contents.get_contents(coll_id=coll_ids, status=status, to_json=to_json, session=session)
+        if relation_type is None:
+            content_relation_type = None
+        else:
+            if relation_type == CollectionRelationType.Output:
+                content_relation_type = ContentRelationType.Output
+            elif relation_type == CollectionRelationType.Input:
+                content_relation_type = ContentRelationType.Input
+            elif relation_type == CollectionRelationType.Log:
+                content_relation_type = ContentRelationType.Log
+        rets = orm_contents.get_contents(coll_id=coll_ids, status=status, to_json=to_json,
+                                         relation_type=content_relation_type, session=session)
     else:
         rets = []
     return rets
@@ -609,3 +619,109 @@ def get_output_contents_by_request_id_status(request_id, name, content_status, l
     if contents and limit and len(contents) > limit:
         contents = contents[:limit]
     return contents
+
+
+@transactional_session
+def add_contents_update(contents, bulk_size=10000, session=None):
+    """
+    Add contents update.
+
+    :param contents: dict of contents.
+    :param session: session.
+
+    :raises DuplicatedObject: If a collection with the same name exists.
+    :raises DatabaseException: If there is a database error.
+
+    :returns: content ids.
+    """
+    return orm_contents.add_contents_update(contents, bulk_size=bulk_size, session=session)
+
+
+@transactional_session
+def delete_contents_update(session=None):
+    """
+    delete a content.
+
+    :param session: The database session in use.
+
+    :raises NoObject: If no content is founded.
+    :raises DatabaseException: If there is a database error.
+    """
+    return orm_contents.delete_contents_update(session=session)
+
+
+def get_contents_ext_maps():
+    return orm_contents.get_contents_ext_maps()
+
+
+@transactional_session
+def add_contents_ext(contents, bulk_size=10000, session=None):
+    """
+    Add contents ext.
+
+    :param contents: dict of contents.
+    :param session: session.
+
+    :raises DuplicatedObject: If a collection with the same name exists.
+    :raises DatabaseException: If there is a database error.
+
+    :returns: content ids.
+    """
+    return orm_contents.add_contents_ext(contents, bulk_size=bulk_size, session=session)
+
+
+@transactional_session
+def update_contents_ext(parameters, session=None):
+    """
+    update contents ext.
+
+    :param parameters: list of dictionary of parameters.
+    :param session: The database session in use.
+
+    :raises NoObject: If no content is founded.
+    :raises DatabaseException: If there is a database error.
+
+    """
+    return orm_contents.update_contents_ext(parameters, session=session)
+
+
+@read_session
+def get_contents_ext(request_id=None, transform_id=None, workload_id=None, coll_id=None, status=None, session=None):
+    """
+    Get content or raise a NoObject exception.
+
+    :param request_id: request id.
+    :param transform_id: transform id.
+    :param workload_id: workload id.
+
+    :param session: The database session in use.
+
+    :raises NoObject: If no content is founded.
+
+    :returns: list of contents.
+    """
+    return orm_contents.get_contents_ext(request_id=request_id, transform_id=transform_id, workload_id=workload_id,
+                                         coll_id=coll_id, status=status, session=session)
+
+
+@read_session
+def get_contents_ext_ids(request_id=None, transform_id=None, workload_id=None, coll_id=None, status=None, session=None):
+    """
+    Get content or raise a NoObject exception.
+
+    :param request_id: request id.
+    :param transform_id: transform id.
+    :param workload_id: workload id.
+
+    :param session: The database session in use.
+
+    :raises NoObject: If no content is founded.
+
+    :returns: list of content ids.
+    """
+    return orm_contents.get_contents_ext_ids(request_id=request_id, transform_id=transform_id, workload_id=workload_id,
+                                             coll_id=coll_id, status=status, session=session)
+
+
+def combine_contents_ext(contents, contents_ext, with_status_name=False):
+    return orm_contents.combine_contents_ext(contents, contents_ext, with_status_name=with_status_name)
