@@ -6,7 +6,7 @@
 # http://www.apache.org/licenses/LICENSE-2.0OA
 #
 # Authors:
-# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2020
+# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2022
 
 
 """
@@ -16,7 +16,7 @@ SQLAlchemy models for idds relational data
 import datetime
 from enum import Enum
 
-from sqlalchemy import BigInteger, Boolean, Column, DateTime, Integer, String, event, DDL, Interval
+from sqlalchemy import BigInteger, Boolean, Column, DateTime, Integer, String, Float, event, DDL, Interval
 from sqlalchemy.ext.compiler import compiles
 # from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import object_mapper
@@ -163,14 +163,23 @@ class Request(BASE, ModelBase):
 
     @property
     def request_metadata(self):
-        if self._request_metadata and 'workflow' in self._request_metadata:
-            workflow = self._request_metadata['workflow']
-            workflow_data = None
-            if self._processing_metadata and 'workflow_data' in self._processing_metadata:
-                workflow_data = self._processing_metadata['workflow_data']
-            if workflow is not None and workflow_data is not None:
-                workflow.metadata = workflow_data
-                self._request_metadata['workflow'] = workflow
+        if self._request_metadata:
+            if 'workflow' in self._request_metadata:
+                workflow = self._request_metadata['workflow']
+                workflow_data = None
+                if self._processing_metadata and 'workflow_data' in self._processing_metadata:
+                    workflow_data = self._processing_metadata['workflow_data']
+                if workflow is not None and workflow_data is not None:
+                    workflow.metadata = workflow_data
+                    self._request_metadata['workflow'] = workflow
+            if 'build_workflow' in self._request_metadata:
+                build_workflow = self._request_metadata['build_workflow']
+                build_workflow_data = None
+                if self._processing_metadata and 'build_workflow_data' in self._processing_metadata:
+                    build_workflow_data = self._processing_metadata['build_workflow_data']
+                if build_workflow is not None and build_workflow_data is not None:
+                    build_workflow.metadata = build_workflow_data
+                    self._request_metadata['build_workflow'] = build_workflow
         return self._request_metadata
 
     @request_metadata.setter
@@ -179,9 +188,13 @@ class Request(BASE, ModelBase):
             self._request_metadata = request_metadata
         if self._processing_metadata is None:
             self._processing_metadata = {}
-        if request_metadata and 'workflow' in request_metadata:
-            workflow = request_metadata['workflow']
-            self._processing_metadata['workflow_data'] = workflow.metadata
+        if request_metadata:
+            if 'workflow' in request_metadata:
+                workflow = request_metadata['workflow']
+                self._processing_metadata['workflow_data'] = workflow.metadata
+            if 'build_workflow' in request_metadata:
+                build_workflow = request_metadata['build_workflow']
+                self._processing_metadata['build_workflow_data'] = build_workflow.metadata
 
     @property
     def processing_metadata(self):
@@ -193,7 +206,7 @@ class Request(BASE, ModelBase):
             self._processing_metadata = {}
         if processing_metadata:
             for k in processing_metadata:
-                if k != 'workflow_data':
+                if k != 'workflow_data' and k != 'build_workflow_data':
                     self._processing_metadata[k] = processing_metadata[k]
 
     def _items_extend(self):
@@ -201,13 +214,22 @@ class Request(BASE, ModelBase):
                 ('processing_metadata', self.processing_metadata)]
 
     def update(self, values, flush=True, session=None):
-        if values and 'request_metadata' in values and 'workflow' in values['request_metadata']:
-            workflow = values['request_metadata']['workflow']
+        if values and 'request_metadata' in values:
+            if 'workflow' in values['request_metadata']:
+                workflow = values['request_metadata']['workflow']
+
+                if workflow is not None:
+                    if 'processing_metadata' not in values:
+                        values['processing_metadata'] = {}
+                    values['processing_metadata']['workflow_data'] = workflow.metadata
+            if 'build_workflow' in values['request_metadata']:
+                build_workflow = values['request_metadata']['build_workflow']
+
+                if build_workflow is not None:
+                    if 'processing_metadata' not in values:
+                        values['processing_metadata'] = {}
+                    values['processing_metadata']['build_workflow_data'] = build_workflow.metadata
 
-            if workflow is not None:
-                if 'processing_metadata' not in values:
-                    values['processing_metadata'] = {}
-                values['processing_metadata']['workflow_data'] = workflow.metadata
         if values and 'request_metadata' in values:
             del values['request_metadata']
         if values and 'processing_metadata' in values:
@@ -472,6 +494,10 @@ class Collection(BASE, ModelBase):
     processing_files = Column(Integer())
     failed_files = Column(Integer())
     missing_files = Column(Integer())
+    ext_files = Column(Integer())
+    processed_ext_files = Column(Integer())
+    failed_ext_files = Column(Integer())
+    missing_ext_files = Column(Integer())
     processing_id = Column(Integer())
     retries = Column(Integer(), default=0)
     created_at = Column("created_at", DateTime, default=datetime.datetime.utcnow)
@@ -500,6 +526,7 @@ class Content(BASE, ModelBase):
     request_id = Column(BigInteger().with_variant(Integer, "sqlite"))
     workload_id = Column(Integer())
     map_id = Column(BigInteger().with_variant(Integer, "sqlite"), default=0)
+    content_dep_id = Column(BigInteger())
     scope = Column(String(SCOPE_LENGTH))
     name = Column(String(LONG_NAME_LENGTH))
     min_id = Column(Integer(), default=0)
@@ -533,9 +560,103 @@ class Content(BASE, ModelBase):
                    CheckConstraint('coll_id IS NOT NULL', name='CONTENTS_COLL_ID_NN'),
                    Index('CONTENTS_STATUS_UPDATED_IDX', 'status', 'locking', 'updated_at', 'created_at'),
                    Index('CONTENTS_ID_NAME_IDX', 'coll_id', 'scope', 'name', 'status'),
+                   Index('CONTENTS_DEP_IDX', 'request_id', 'transform_id', 'content_dep_id'),
                    Index('CONTENTS_REQ_TF_COLL_IDX', 'request_id', 'transform_id', 'coll_id', 'status'))
 
 
+class Content_update(BASE, ModelBase):
+    """Represents a content update"""
+    __tablename__ = 'contents_update'
+    content_id = Column(BigInteger().with_variant(Integer, "sqlite"), primary_key=True)
+    substatus = Column(EnumWithValue(ContentStatus))
+
+
+class Content_ext(BASE, ModelBase):
+    """Represents a content extension"""
+    __tablename__ = 'contents_ext'
+    content_id = Column(BigInteger().with_variant(Integer, "sqlite"), primary_key=True)
+    transform_id = Column(BigInteger().with_variant(Integer, "sqlite"))
+    coll_id = Column(BigInteger().with_variant(Integer, "sqlite"))
+    request_id = Column(BigInteger().with_variant(Integer, "sqlite"))
+    workload_id = Column(Integer())
+    map_id = Column(BigInteger().with_variant(Integer, "sqlite"), default=0)
+    status = Column(EnumWithValue(ContentStatus))
+    panda_id = Column(BigInteger())
+    job_definition_id = Column(BigInteger())
+    scheduler_id = Column(String(128))
+    pilot_id = Column(String(200))
+    creation_time = Column(DateTime)
+    modification_time = Column(DateTime)
+    start_time = Column(DateTime)
+    end_time = Column(DateTime)
+    prod_source_label = Column(String(20))
+    prod_user_id = Column(String(250))
+    assigned_priority = Column(Integer())
+    current_priority = Column(Integer())
+    attempt_nr = Column(Integer())
+    max_attempt = Column(Integer())
+    max_cpu_count = Column(Integer())
+    max_cpu_unit = Column(String(32))
+    max_disk_count = Column(Integer())
+    max_disk_unit = Column(String(10))
+    min_ram_count = Column(Integer())
+    min_ram_unit = Column(String(10))
+    cpu_consumption_time = Column(Integer())
+    cpu_consumption_unit = Column(String(128))
+    job_status = Column(String(10))
+    job_name = Column(String(255))
+    trans_exit_code = Column(Integer())
+    pilot_error_code = Column(Integer())
+    pilot_error_diag = Column(String(500))
+    exe_error_code = Column(Integer())
+    exe_error_diag = Column(String(500))
+    sup_error_code = Column(Integer())
+    sup_error_diag = Column(String(250))
+    ddm_error_code = Column(Integer())
+    ddm_error_diag = Column(String(500))
+    brokerage_error_code = Column(Integer())
+    brokerage_error_diag = Column(String(250))
+    job_dispatcher_error_code = Column(Integer())
+    job_dispatcher_error_diag = Column(String(250))
+    task_buffer_error_code = Column(Integer())
+    task_buffer_error_diag = Column(String(300))
+    computing_site = Column(String(128))
+    computing_element = Column(String(128))
+    grid = Column(String(50))
+    cloud = Column(String(50))
+    cpu_conversion = Column(Float())
+    task_id = Column(BigInteger())
+    vo = Column(String(16))
+    pilot_timing = Column(String(100))
+    working_group = Column(String(20))
+    processing_type = Column(String(64))
+    prod_user_name = Column(String(60))
+    core_count = Column(Integer())
+    n_input_files = Column(Integer())
+    req_id = Column(BigInteger())
+    jedi_task_id = Column(BigInteger())
+    actual_core_count = Column(Integer())
+    max_rss = Column(Integer())
+    max_vmem = Column(Integer())
+    max_swap = Column(Integer())
+    max_pss = Column(Integer())
+    avg_rss = Column(Integer())
+    avg_vmem = Column(Integer())
+    avg_swap = Column(Integer())
+    avg_pss = Column(Integer())
+    max_walltime = Column(Integer())
+    disk_io = Column(Integer())
+    failed_attempt = Column(Integer())
+    hs06 = Column(Integer())
+    hs06sec = Column(Integer())
+    memory_leak = Column(String(10))
+    memory_leak_x2 = Column(String(10))
+    job_label = Column(String(20))
+
+    _table_args = (PrimaryKeyConstraint('content_id', name='CONTENTS_EXT_PK'),
+                   Index('CONTENTS_EXT_RTF_IDX', 'request_id', 'transform_id', 'workload_id', 'coll_id', 'content_id', 'panda_id', 'status'))
+
+
 class Health(BASE, ModelBase):
     """Represents the status of the running agents"""
     __tablename__ = 'health'
@@ -596,7 +717,7 @@ class Command(BASE, ModelBase):
     status = Column(EnumWithValue(CommandStatus))
     substatus = Column(Integer())
     locking = Column(EnumWithValue(CommandLocking))
-    username = Column(String(20))
+    username = Column(String(50))
     retries = Column(Integer(), default=0)
     source = Column(EnumWithValue(CommandLocation))
     destination = Column(EnumWithValue(CommandLocation))
@@ -617,7 +738,7 @@ def register_models(engine):
     """
 
     # models = (Request, Workprogress, Transform, Workprogress2transform, Processing, Collection, Content, Health, Message)
-    models = (Request, Transform, Processing, Collection, Content, Health, Message, Command)
+    models = (Request, Transform, Processing, Collection, Content, Content_update, Content_ext, Health, Message, Command)
 
     for model in models:
         # if not engine.has_table(model.__tablename__, model.metadata.schema):
@@ -630,7 +751,18 @@ def unregister_models(engine):
     """
 
     # models = (Request, Workprogress, Transform, Workprogress2transform, Processing, Collection, Content, Health, Message)
-    models = (Request, Transform, Processing, Collection, Content, Health, Message, Command)
+    models = (Request, Transform, Processing, Collection, Content, Content_update, Content_ext, Health, Message, Command)
 
     for model in models:
         model.metadata.drop_all(engine)   # pylint: disable=maybe-no-member
+
+
+@event.listens_for(Content_update.__table__, "after_create")
+def _update_content_dep_status(target, connection, **kw):
+    DDL("""
+        CREATE TRIGGER update_content_dep_status BEFORE DELETE ON contents_update
+        for each row
+        BEGIN
+            UPDATE contents set substatus = :old.substatus where contents.content_dep_id = :old.content_id;
+        END;
+    """)
