@@ -35,8 +35,13 @@ def migrate():
     # doma google
     doma_google_host = 'https://34.133.138.229:443/idds'  # noqa F841
 
-    cm1 = ClientManager(host=atlas_host)
+    slac_k8s_dev_host = 'https://rubin-panda-idds-dev.slac.stanford.edu:8443/idds'  # noqa F841
+
+    cern_k8s_dev_host = 'https://panda-idds-dev.cern.ch/idds'  # noqa F841
+
+    # cm1 = ClientManager(host=atlas_host)
     cm1 = ClientManager(host=doma_host)
+    # cm1 = ClientManager(host=slac_k8s_dev_host)
     # reqs = cm1.get_requests(request_id=290)
     # old_request_id = 298163
     # old_request_id = 350723
@@ -47,16 +52,20 @@ def migrate():
     old_request_id = 372930
     old_request_id = 2603
     old_request_id = 2802
+    old_request_id = 2816
 
+    # old_request_id = 1
     # for old_request_id in [152]:
     # for old_request_id in [60]:    # noqa E115
     # for old_request_id in [200]:    # noqa E115
     for old_request_id in [old_request_id]:    # noqa E115  # doma 183
         reqs = cm1.get_requests(request_id=old_request_id, with_metadata=True)
 
-        cm2 = ClientManager(host=dev_host)
-        cm2 = ClientManager(host=doma_host)
+        # cm2 = ClientManager(host=dev_host)
+        # cm2 = ClientManager(host=doma_host)
         # cm2 = ClientManager(host=atlas_host)
+        cm2 = ClientManager(host=slac_k8s_dev_host)
+        # cm2 = ClientManager(host=cern_k8s_dev_host)
         # print(reqs)
 
         print("num requests: %s" % len(reqs))
